package gov.va.med.mhv.login.hook;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.stereotype.Component;

import com.liferay.portal.kernel.events.Action;
import com.liferay.portal.kernel.events.ActionException;
import com.liferay.portal.security.auth.AuthException;

import gov.va.med.mhv.common.api.exception.MHVException;
import gov.va.med.mhv.login.hook.SystemDownException.ErrorCodeEnum;
import gov.va.med.mhv.usermgmt.common.dto.ActivityDTO;
import gov.va.med.mhv.usermgmt.common.enums.ActivityActionTypeEnumeration;
import gov.va.med.mhv.usermgmt.common.enums.ActivityActorTypeEnumeration;
import gov.va.med.mhv.usermgmt.common.enums.PerformerTypeEnumeration;
import gov.va.med.mhv.usermgmt.service.AccountActivityCreatorService;

@Component
public class PreLogoutAction extends Action {
	private static Logger logger = LogManager.getLogger(PreLogoutAction.class);

	private static final String API_SESSION_EXP_KEY = "APISessionExpiration";
	private static final String RFC1123_PATTERN  = "EEE, dd MMM yyyy HH:mm:ss z";
	private static final String CSID_PARAM = "va_eauth_csid";

	private AccountActivityCreatorService accountActivityCreatorService;

	/*
	 *  when user logs out a record is inserted in to Activity table for audit purpose
	 */
	@Override
	public void run(HttpServletRequest request, HttpServletResponse response)
			throws ActionException {
		try {
			HttpSession session = request.getSession();
			
			// determine if session timeout
			Date expiresDt = null;
			String expires = (String)session.getAttribute(API_SESSION_EXP_KEY);
			boolean sessionExpired = false;
			
			if( expires != null ) {
				SimpleDateFormat sdf = new SimpleDateFormat(RFC1123_PATTERN);
				try {
					expiresDt = sdf.parse(expires);
				} catch (ParseException e) {
					logger.error("API Session Expiration is invalid from http session", e);
				}
			}
			
			if( expiresDt != null && expiresDt.before(new Date()) ) {
				sessionExpired = true;
			}
			
			String username=null;
			Long userProfileId=null;
			
			if(session.getAttribute("LIFERAY_SHARED_userid") != null){
				username = (String)session.getAttribute("LIFERAY_SHARED_userid");
			}
			if(session.getAttribute("LIFERAY_SHARED_userprofileid") != null){
				userProfileId= (Long)session.getAttribute("LIFERAY_SHARED_userprofileid");
			}

			// Remove requestURL session attribute that is being utilized in CustomPreSessionExpireAction
			session.removeAttribute("requestURL");
			
			String requestUrl = request.getRequestURL().toString();
			session.setAttribute("requestURL", requestUrl);
						
			// if user logged in from DSLOGON, there will be a request header of "va_eauth_csid", otherwise its normal MHV logon
			String csid = request.getHeader(CSID_PARAM);
			boolean dsLogon = (csid != null);

			logger.debug("userProfileId=" + userProfileId + ", requestUrl=" + requestUrl
					+ ", csid=" + csid + ", dsLogon=" + dsLogon + ", sessionExpired=" + sessionExpired);
			
			// create AAL entry for logout or session timeout
			if (username != null && userProfileId != null) {
				// Need to check if delegation exists 
				if ((session.getAttribute("showDelegateUserText") != null) && (boolean)session.getAttribute("showDelegateUserText")) {
					if (sessionExpired) {
						createTimeoutActivityForDelegation(username,userProfileId,request);
					} else {
						if (dsLogon) {
							createLogoutActivityForDelegation(username,userProfileId,request, ActivityActionTypeEnumeration.LOGOUT_WITH_DSLOGON);
						} else {
							createLogoutActivityForDelegation(username,userProfileId,request, ActivityActionTypeEnumeration.LOGOUT_WITH_MY_HEALTHEVET);
						}
					}
				} else if (requestUrl != null && requestUrl.contains("portal/logout")) {
					if (sessionExpired) {
						createTimeoutActivity(username,userProfileId);
					} else {
						if (dsLogon) {
							createLogoutActivity(username,userProfileId, ActivityActionTypeEnumeration.LOGOUT_WITH_DSLOGON);
						} else {
							createLogoutActivity(username,userProfileId, ActivityActionTypeEnumeration.LOGOUT_WITH_MY_HEALTHEVET);
						}
					}
				}
			}

			String sessionId = session.getId();
			session.invalidate();
			Cookie[] cookies = request.getCookies();
		    for (Cookie cookie : cookies) {
		      if (sessionId.equalsIgnoreCase(cookie.getValue())) {
		        cookie.setMaxAge(0);
		        cookie.setValue(null);
		        cookie.setDomain(request.getServerName());
		        cookie.setPath(request.getServletContext().getContextPath() + "/");
		        cookie.setSecure(request.isSecure());
		        response.addCookie(cookie);
		        break;
		      }
		    }

		} catch (Exception e) {			
			logger.error("Error in PreLogoutAction", e);
		}
	}
	
	private void createTimeoutActivityForDelegation(String username, Long userProfileId, HttpServletRequest request) {
		logger.debug("Creating Account Activity Log - createLogoutActivityForDelegation");	

		try {
			ActivityDTO timeoutActivity = new ActivityDTO();
			timeoutActivity.setAction(ActivityActionTypeEnumeration.SESSION_TIMEOUT.getDescription());
			timeoutActivity.setActivityType("Logout");
			timeoutActivity.setUserProfileId(userProfileId);
			timeoutActivity.setStatus(Boolean.TRUE);
			timeoutActivity.setPerformerType(ActivityActorTypeEnumeration.HEALTHCARE_PROXY_FULL.getDescription());
			timeoutActivity.setDetailValue("Performed by Delegate: "+(String) request.getSession().getAttribute("LIFERAY_SHARED_userid_delegatee_fullname"));
			timeoutActivity.setCompletionTime(new Date());

			getAccountActivityCreatorService().createAccountActivityLog(timeoutActivity);
		} catch (MHVException e) {
			logger.error("Error Creating Account Activity Log for Delegation - Session timeout", e);
			throw new RuntimeException("Error Creating Account Activity Log for Delegation - Session timeout", e);
		}
	}

	// AAL entry for MHV Timeout
	private void createTimeoutActivity(String username, Long userProfileId) throws AuthException{
		
		logger.debug("Creating Account Activity Log - createTimeoutActivity");	
		
		try {
			ActivityDTO timeoutActivity = new ActivityDTO();
			timeoutActivity.setAction(ActivityActionTypeEnumeration.SESSION_TIMEOUT.getDescription());
			timeoutActivity.setActivityType("Logout");
			timeoutActivity.setUserProfileId(userProfileId);
			timeoutActivity.setStatus(Boolean.TRUE);
			timeoutActivity.setPerformerType(PerformerTypeEnumeration.valueOf("System").toString());
			timeoutActivity.setDetailValue("None");
			timeoutActivity.setCompletionTime(new Date());
			
			getAccountActivityCreatorService().createAccountActivityLog(timeoutActivity);
			
		} catch (MHVException e) {
			logger.error("Error Creating Account Activity Log - Session timeout", e);
			throw new AuthException("Error Creating Account Activity Log Service.", new SystemDownException(ErrorCodeEnum.USER_MANAGMENT_FAILURE_POST_LOGIN_UPDATE, e));			
		}
	}	

	private void createLogoutActivityForDelegation(String username, Long userProfileId, HttpServletRequest request, ActivityActionTypeEnumeration activityActionType) {
		logger.debug("Creating Account Activity Log - createLogoutActivityForDelegation");	

		try {
			ActivityDTO logoutActivity = new ActivityDTO();
			logoutActivity.setAction(activityActionType.getDescription());
			logoutActivity.setActivityType("Logout");
			logoutActivity.setUserProfileId(userProfileId);
			logoutActivity.setStatus(Boolean.TRUE);
			logoutActivity.setPerformerType(ActivityActorTypeEnumeration.HEALTHCARE_PROXY_FULL.getDescription());
			logoutActivity.setDetailValue("Performed by Delegate: "+(String) request.getSession().getAttribute("LIFERAY_SHARED_userid_delegatee_fullname"));
			logoutActivity.setCompletionTime(new Date());

			getAccountActivityCreatorService().createAccountActivityLog(logoutActivity);
		} catch (MHVException e) {
			logger.error("Error Creating Account Activity Log for Delegation - Logout", e);
			throw new RuntimeException("Error Creating Account Activity Log for Delegation - Logout", e);
		}
	}

	private void createLogoutActivity(String username, Long userProfileId, ActivityActionTypeEnumeration activityActionType){
		logger.debug("Creating Account Activity Log - createLogoutActivity");	

		try {
			ActivityDTO logoutActivity = new ActivityDTO();
			logoutActivity.setAction(activityActionType.getDescription());
			logoutActivity.setActivityType("Logout");
			logoutActivity.setUserProfileId(userProfileId);
			logoutActivity.setStatus(Boolean.TRUE);
			logoutActivity.setPerformerType(PerformerTypeEnumeration.valueOf("Self").toString());
			logoutActivity.setDetailValue("None");
			logoutActivity.setCompletionTime(new Date());

			getAccountActivityCreatorService().createAccountActivityLog(logoutActivity);
		} catch (MHVException e) {
			// TODO: Figure out if this is really the right thing to do here.
			// TODO: Throw runtime exception???
			logger.error("Error Creating Account Activity Log - createLogoutActivity", e);
			throw new RuntimeException("Error Creating Account Activity Log - createLogoutActivity", e);
		}
	}

	private AccountActivityCreatorService getAccountActivityCreatorService() {
		if(this.accountActivityCreatorService == null) {
			this.accountActivityCreatorService = (AccountActivityCreatorService)ApplicationContextProvider.getApplicationContext().getBean("activityProxy");
		}
		return this.accountActivityCreatorService;
	}
}
