package gov.va.med.mhv.usermgmt.batch.listener;

import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.time.Duration;
import java.util.Collection;
import java.util.Date;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.batch.core.BatchStatus;
import org.springframework.batch.core.JobExecution;
import org.springframework.batch.core.JobExecutionListener;
import org.springframework.batch.core.StepExecution;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import gov.va.med.mhv.usermgmt.batch.processor.HealthSummaryProcessor;
import gov.va.med.mhv.usermgmt.service.EmailFailureException;
import gov.va.med.mhv.usermgmt.service.EmailService;

@Component
public class HealthSummaryJobListener implements JobExecutionListener {
	private static final Logger logger = LogManager.getLogger(HealthSummaryJobListener.class);
	
	// US04: 1.2 email - batch job failed 		
	@Value("${health.summary.email.failed.job.from}")
	private String emailFailedJobFrom;
	
	@Value("${health.summary.email.failed.job.from.name}")
	private String emailFailedJobFromName;

	@Value("${health.summary.email.failed.job.to}")
	private String emailFailedJobTo;

	@Value("${health.summary.email.failed.job.cc}")
	private String emailFailedJobCc;

	@Value("${health.summary.email.failed.job.bcc}")
	private String emailFailedJobBcc;

	@Value("${health.summary.email.failed.job.subject}")
	private String emailFailedJobSubject;

	@Value("${health.summary.email.failed.job.body}")
	private String emailFailedJobBody;
	
	@Autowired
	private EmailService emailService;
	
	@Autowired
	private HealthSummaryProcessor healthSummaryProcessor;
	
	@Override
	public void afterJob(JobExecution jobExecution) {
		logger.info("afterJob " + jobExecution.toString());
		
		writeLogFile(jobExecution);
		
		if (jobExecution.getStatus() == BatchStatus.FAILED) {

			logger.info("Sending email Notification: Health Summary Batch Job Failed");

			try {
				emailService.send(emailFailedJobFrom, emailFailedJobFromName, emailFailedJobSubject,
						emailFailedJobBody, emailFailedJobTo, emailFailedJobCc, emailFailedJobBcc, null);
				
			} catch (EmailFailureException e) {
				logger.error("Failed to send notification email", e);
				jobExecution.addFailureException(e);
			}
		}
	}

	@Override
	public void beforeJob(JobExecution jobExecution) {
		logger.info("afterJob " + jobExecution.toString());
		
		healthSummaryProcessor.resetLogCounts();
	}
	
	private void writeLogFile(JobExecution jobExecution) {
    	SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
    	Date fileDate = new Date();
    	String fileName = "health-summary-log-" + sdf.format(fileDate) + ".txt";
    	FileWriter fw = null;
    	
    	try {
    		logger.debug("sending output to log file='" + fileName + "'");
    		
			fw = new FileWriter(fileName, true);
			
			String jobDuration = "";
			long duration = jobExecution.getEndTime().getTime() - jobExecution.getStartTime().getTime();
			
			if (duration < 1000) {
				jobDuration = "< 1 second";
			} else {
				Duration d = Duration.ofMillis(duration);
				jobDuration = String.format("%02d:%02d:%02d", d.toHours(), d.toMinutes(), d.toMillis() / 1000);
			}
			
			Collection<StepExecution> coll = jobExecution.getStepExecutions();
			
			fw.write(String.format("%n**********************************************%n"));

			fw.write(String.format("Job Id:\t%s%n", jobExecution.getJobId()));
			fw.write(String.format("Job Start:\t%s%n", jobExecution.getStartTime()));
			fw.write(String.format("Job End:\t%s%n", jobExecution.getEndTime()));
			fw.write(String.format("Job Duration:\t%s (hh:mm:ss)%n", jobDuration));
			fw.write(String.format("Job Status:\t%s%n", jobExecution.getExitStatus().getExitCode()));
			fw.write(String.format("Row Count:\t%d%n", healthSummaryProcessor.getRowCount()));
			fw.write(String.format("%n"));
			fw.write(String.format("Emails sent successful: %d%n", healthSummaryProcessor.getLogEmailsSentSuccess()));
			fw.write(String.format("Emails sent not successful: %d%n", healthSummaryProcessor.getLogEmailsSentNotSuccess()));
			fw.write(String.format("Emails skipped, no valid contact email address: %d%n", healthSummaryProcessor.getLogEmailsSkippedNoAddress()));
			fw.write(String.format("Emails skipped, user email notification preferences is off: %d%n", healthSummaryProcessor.getLogNotificationsOff()));
			fw.write(String.format("VA Direct connection errors: %d%n", healthSummaryProcessor.getLogVADirectErrors()));
			
			fw.write(String.format("%nStep Execution Details:%n"));
			for (StepExecution step : coll) {
				fw.write(step.getSummary());
				fw.write(String.format("%n"));
			}
			
		} catch (IOException e) {
			logger.error("could not write to log file='" + fileName + "' ", e);
		} finally {
			try {
				if (fw != null) {
					fw.close();
				}
			} catch (IOException e) {
				logger.error("could not close log file='" + fileName + "' ", e);
			}
		}
	}
	

	@Override
	public String toString() {
		return "HealthSummaryJobListener [emailFailedJobFrom=" + emailFailedJobFrom + ", emailFailedJobFromName="
				+ emailFailedJobFromName + ", emailFailedJobTo=" + emailFailedJobTo + ", emailFailedJobCc="
				+ emailFailedJobCc + ", emailFailedJobBcc=" + emailFailedJobBcc + ", emailFailedJobSubject="
				+ emailFailedJobSubject + ", emailFailedJobBody=" + emailFailedJobBody + "]";
	}

}
