package gov.va.med.mhv.usermgmt.batch.processor;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.batch.core.ExitStatus;
import org.springframework.batch.core.StepExecution;
import org.springframework.batch.core.StepExecutionListener;
import org.springframework.batch.item.ItemProcessor;

import com.itextpdf.text.DocumentException;

import gov.va.med.mhv.common.api.util.ResponseUtil;
import gov.va.med.mhv.common.data.model.Patient;
import gov.va.med.mhv.usermgmt.report.MviPdfReport;
import gov.va.med.mhv.usermgmt.service.MviIntegrationService;
import gov.va.med.mhv.usermgmt.util.DetermineMatchCounter;

public class AccountSyncMHVMVIMisMatchProcessor implements ItemProcessor<Patient, Patient>, StepExecutionListener {
	private static final Logger logger = LogManager.getLogger(AccountSyncMHVMVIMisMatchProcessor.class);

	private MviIntegrationService mviIntegrationService;
	private DetermineMatchCounter determineMatchCounter;
	private int partition = -1;

	// mhv and mvi data don't match 4 or more traits
	private MviPdfReport mhvMviMismatchReport = new MviPdfReport();
	private String mhvMviMismatchReportName;
	private String mhvMviMismatchReportTitle;

	// icn missing from mvi
	private MviPdfReport icnMissingReport = new MviPdfReport();
	private String icnMissingReportName;
	private String icnMissingReportTitle;

	@Override
	public Patient process(Patient patient) throws Exception {
		ResponseUtil<Patient> response = new ResponseUtil<Patient>();
		Patient mviPatient = null;

		if (patient.getUserProfileId() != 804677L) {
			return null;
		}
		
		try {

			mviIntegrationService.searchPersonInMVIByICNAccountSyncCron(patient, response);

//			if (response.getPojoObject() == null) {
//				// record not matched by ICN
//				mviIntegrationService.searchPersonInMVI(patient.getUserProfile(), false, response, null);
//
//				if (response.getPojoObject() == null) {
//					// ICN missing from MVI
//					icnMissingReport.write(patient);
//					return patient;
//				}
//			}

			mviPatient = (Patient) response.getPojoObject();

			if (logger.isDebugEnabled()) {
				try {
					if (mviPatient != null) {
					logger.debug(String.format("patient id=%d, userProfileId=%d", patient.getId(),
							patient.getUserProfileId()));
					logger.debug(String.format("firstName: %s %s", patient.getUserProfile().getFirstName(),
							mviPatient.getUserProfile().getFirstName()));
					logger.debug(String.format("middleName: %s %s", patient.getUserProfile().getMiddleName(),
							mviPatient.getUserProfile().getMiddleName()));
					logger.debug(String.format("lastName: %s %s", patient.getUserProfile().getLastName(),
							mviPatient.getUserProfile().getLastName()));
					logger.debug(String.format("gender: %s %s", patient.getUserProfile().getGender(),
							mviPatient.getUserProfile().getGender()));
					logger.debug(String.format("dob: %s %s", patient.getUserProfile().getBirthDate(),
							mviPatient.getUserProfile().getBirthDate()));
					logger.debug(String.format("ssn: %s %s", patient.getUserProfile().getSsn(),
							mviPatient.getUserProfile().getSsn()));
					logger.debug(String.format("icn: %s %s", patient.getIcn(), mviPatient.getIcn()));
					} else {
						logger.debug("MVI patient is null");
					}
					
				} catch (Exception e) {
					logger.error(e);
				}
			}
			
			if (mviPatient == null) {
				// not in mvi
				icnMissingReport.write(patient);
			} else if (mviPatient.getIcn() == null || mviPatient.getIcn().length() == 0) {
				// ICN missing from MVI
				icnMissingReport.write(patient);
			} else if (patient.getIcn().compareToIgnoreCase(mviPatient.getIcn()) != 0) {
				// ICN is different in mhv and mvi
				mhvMviMismatchReport.write(patient);
			} else {
				// determine number of traits in common (first name, last name,
				// middle name,
				// SSN, DOB, Gender)
				int matchCounter = determineMatchCounter.determineMatchCounter(patient, mviPatient);

				if (matchCounter >= 4) {
					// do nothing
				} else if (matchCounter < 4) {
					// less than 4 traits in common
					mhvMviMismatchReport.write(patient);
				}
			}

		} catch (Exception e) {
			logger.error(e);
		}

		return patient;
	}

	@Override
	public ExitStatus afterStep(StepExecution stepExecution) {
		logger.debug("afterStep");

		try {
			mhvMviMismatchReport.close(stepExecution.getReadCount());
		} catch (DocumentException e) {
			logger.error("Error closing mhvmviMismatchReport pdf report ", e);
		}

		try {
			icnMissingReport.close(stepExecution.getReadCount());
		} catch (DocumentException e) {
			logger.error("Error closing icnMissingReport pdf report ", e);
		}

		return stepExecution.getExitStatus();
	}

	@Override
	public void beforeStep(StepExecution stepExecution) {

		if (logger.isDebugEnabled()) {
			logger.debug(
					String.format("beforeStep: thread=%d, stepName=%s, partition=%d", Thread.currentThread().getId(),
							stepExecution.getStepName(), stepExecution.getExecutionContext().getInt("partition", -1)));
		}

		partition = stepExecution.getExecutionContext().getInt("partition", -1);

		try {
			mhvMviMismatchReport.create(mhvMviMismatchReportTitle, mhvMviMismatchReportName, partition,
					stepExecution.getJobExecution().getStartTime());
		} catch (DocumentException e) {
			logger.error("Error creating mhvmviMismatchReport pdf report ", e);
		}

		try {
			icnMissingReport.create(icnMissingReportTitle, icnMissingReportName, partition,
					stepExecution.getJobExecution().getStartTime());
		} catch (DocumentException e) {
			logger.error("Error creating icnMissingReport pdf report ", e);
		}
	}

	public MviIntegrationService getMviIntegrationService() {
		return mviIntegrationService;
	}

	public void setMviIntegrationService(MviIntegrationService mviIntegrationService) {
		this.mviIntegrationService = mviIntegrationService;
	}

	public DetermineMatchCounter getDetermineMatchCounter() {
		return determineMatchCounter;
	}

	public void setDetermineMatchCounter(DetermineMatchCounter determineMatchCounter) {
		this.determineMatchCounter = determineMatchCounter;
	}

	public String getMhvMviMismatchReportName() {
		return mhvMviMismatchReportName;
	}

	public void setMhvMviMismatchReportName(String mhvMviMismatchReportName) {
		this.mhvMviMismatchReportName = mhvMviMismatchReportName;
	}

	public String getIcnMissingReportName() {
		return icnMissingReportName;
	}

	public void setIcnMissingReportName(String icnMissingReportName) {
		this.icnMissingReportName = icnMissingReportName;
	}

	public String getMhvMviMismatchReportTitle() {
		return mhvMviMismatchReportTitle;
	}

	public void setMhvMviMismatchReportTitle(String mhvMviMismatchReportTitle) {
		this.mhvMviMismatchReportTitle = mhvMviMismatchReportTitle;
	}

	public String getIcnMissingReportTitle() {
		return icnMissingReportTitle;
	}

	public void setIcnMissingReportTitle(String icnMissingReportTitle) {
		this.icnMissingReportTitle = icnMissingReportTitle;
	}
}
