package gov.va.med.mhv.usermgmt.batch.processor;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.batch.core.StepExecution;
import org.springframework.batch.core.annotation.AfterStep;
import org.springframework.batch.item.ItemProcessor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import com.itextpdf.text.DocumentException;

import gov.va.med.mhv.common.api.dto.FacilityInfoDTO;
import gov.va.med.mhv.common.api.dto.InPersonAuthenticationDTOLight;
import gov.va.med.mhv.common.api.exception.MHVException;
import gov.va.med.mhv.common.api.util.ResponseUtil;
import gov.va.med.mhv.common.data.converter.FacilityInfoConveter;
import gov.va.med.mhv.common.data.model.FacilityInfo;
import gov.va.med.mhv.common.data.model.InPersonAuthentication;
import gov.va.med.mhv.common.data.model.Patient;
import gov.va.med.mhv.usermgmt.common.enums.ActivityActorTypeEnumeration;
import gov.va.med.mhv.usermgmt.report.PdfCronJobReportCreatorMap;
import gov.va.med.mhv.usermgmt.service.FacilityInfoService;
import gov.va.med.mhv.usermgmt.service.InPersonAuthenticationService;
import gov.va.med.mhv.usermgmt.service.MviIntegrationService;
import gov.va.med.mhv.usermgmt.service.PatientService;
import gov.va.med.mhv.usermgmt.service.handler.MviProperties;

@Component
public class CorrelateMatchedPatientProcessor implements ItemProcessor<Patient, Patient> {
	private static Logger logger = LogManager.getLogger(CorrelateMatchedPatientProcessor.class);

	private String ERROR_CODE_0 = "0";
	private String ERROR_CODE_1 = "1";
	private String ERROR_CODE_2 = "2";
	private String ERROR_CODE_3 = "3";
	private String ERROR_CODE_4 = "4";
	private String ERROR_CODE_5 = "5";

	@Value("${mvi.correlate.error.records}")
	private String correlatedPatientErrorRecordsBatchReport;

	@Value("${mvi.correlate.records}")
	private String correlatedPatientRecordsBatchReport;

	@Autowired
	private InPersonAuthenticationService inPersonAuthenticationService;

	@Autowired
	private PatientService patientService;

	@Autowired
	private MviIntegrationService mviIntegrationService;

	@Autowired
	private MviProperties mviProperties;

	@Autowired
	private PdfCronJobReportCreatorMap pdfCronJobReportCreatorMap;

	@Autowired
	private FacilityInfoService facilityInfoService;

	private Map<Patient, String> correlatedPatientErrorRecords = new HashMap<Patient, String>();
	private Map<Patient, String> correlatedPatientRecords = new HashMap<Patient, String>();
	private int totalCounter = 0;

	@Override
	public Patient process(Patient patient) throws Exception {
		ResponseUtil responseUtil = new ResponseUtil();

		if (logger.isDebugEnabled()) {
			logger.debug(String.format("Processing Correlate Matched Patient Batch for patient: %s", patient.getId()));
		}

		try {
			
			patientService.updatePatientRegistryInformation(patient, responseUtil);
			if (responseUtil.isFailure()) {
				addToError(patient, ERROR_CODE_1);
				return null;
			}
			
			// TODO: wrong function should be addPersonForCorrelationForCronjob in mviAsynchIntegrationService
			mviIntegrationService.addPersonForCorrelationForIPA(patient, patient.getUserProfile(),
					ActivityActorTypeEnumeration.SYSTEM.toString(), responseUtil, null);
			if (responseUtil.isFailure()) {
				addToError(patient, ERROR_CODE_2);
				return null;
			}
			
			mviIntegrationService.matchPersonInIPA(patient.getUserProfile(), responseUtil, mviProperties, null);
			if (responseUtil.isFailure()) {
				addToError(patient, ERROR_CODE_3);
				return null;
			}
			
			inPersonAuthenticationService.getIPAPatientById(patient.getId(), responseUtil);
			if (responseUtil.isSuccess()) {
				InPersonAuthentication ipa = (InPersonAuthentication) responseUtil.getPojoObject();
				InPersonAuthenticationDTOLight ipaLight = new InPersonAuthenticationDTOLight();
				ipaLight.setVideoViewed(ipa.getVideoViewed());
				ipaLight.setIdentificationPresented(ipa.getIdentificationPresented());
				ipaLight.setParticipationFormSigned(ipa.getParticipationFormSigned());
				ipaLight.setApprovedForRecordsAccess(ipa.getApprovedForRecordsAccess());
				FacilityInfoDTO fidto = new FacilityInfoDTO();
				try {
					fidto = getAustinMHVFacilityInfo();
				} catch (MHVException e) {
					logger.error(e);
				}
				ipaLight.setAuthenticatingFacilityDTO(fidto);
				ipaLight.setDefermentReason(ipa.getDefermentReason());
				ipaLight.setSignedFormOnlineDateTime(ipa.getSignedFormOnlineDateTime());
				ipaLight.setAuthenticatedBy(ipa.getAuthenticatedBy());
				ipaLight.setRemovalReason(null);
				ipaLight.setPatientId(patient.getUserProfileId());
				ipaLight.setParticipationFormSigned(true);
				ipaLight.setIdentificationPresented(true);
				ipaLight.setOplock(0);
				ipaLight.setId(ipa.getId());
				inPersonAuthenticationService.authenticateFull(ipaLight, responseUtil);
				correlatedPatientRecords.put(patient, ERROR_CODE_0);
				totalCounter++;
				return patient;
			} else {
				addToError(patient, ERROR_CODE_4);
				return null;
			}
		} catch (Exception e) {
			correlatedPatientErrorRecords.put(patient, ERROR_CODE_5);
			totalCounter++;
			return null;
		}
	}

	@AfterStep
	public void createPdfReport(StepExecution stepExecution) {
		try {
			if (correlatedPatientErrorRecords != null) {
				if (correlatedPatientErrorRecords.size() > 0) {
					pdfCronJobReportCreatorMap.createPdfReport(correlatedPatientErrorRecords,
							correlatedPatientErrorRecordsBatchReport, totalCounter,
							correlatedPatientErrorRecords.size());
				} else {
					pdfCronJobReportCreatorMap.createPdfReport(correlatedPatientErrorRecords,
							correlatedPatientErrorRecordsBatchReport, totalCounter, 0);
				}
			}
			if (correlatedPatientRecords != null) {
				if (correlatedPatientRecords.size() > 0) {
					pdfCronJobReportCreatorMap.createPdfReport(correlatedPatientRecords,
							correlatedPatientRecordsBatchReport, totalCounter, correlatedPatientRecords.size());
				} else {
					pdfCronJobReportCreatorMap.createPdfReport(correlatedPatientRecords,
							correlatedPatientRecordsBatchReport, totalCounter, 0);
				}
			}
		} catch (DocumentException e) {
			logger.error(e);
		}
		correlatedPatientErrorRecords.clear();
		correlatedPatientRecords.clear();
		totalCounter = 0;
	}

	private void addToError(Patient patient, String errorCode) {

		correlatedPatientErrorRecords.put(patient, errorCode);
		totalCounter++;
	}

	public FacilityInfoDTO getAustinMHVFacilityInfo() throws MHVException {
		List<FacilityInfo> facInfoList = facilityInfoService.queryByStationNumber("200MH");
		if (facInfoList != null && facInfoList.size() > 0) {
			FacilityInfo fi = facInfoList.get(0);
			return new FacilityInfoConveter().convertDomainFacilityInfo(fi);
		}
		return null;
	}

}