package gov.va.med.mhv.usermgmt.batch.processor;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.batch.core.StepExecution;
import org.springframework.batch.core.annotation.AfterStep;
import org.springframework.batch.item.ItemProcessor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import gov.va.med.mhv.common.api.util.ResponseUtil;
import gov.va.med.mhv.common.data.model.InPersonAuthentication;
import gov.va.med.mhv.common.data.model.Patient;
import gov.va.med.mhv.common.data.model.UserProfile;
import gov.va.med.mhv.usermgmt.common.enums.AuthenticationStatusEnumeration;
import gov.va.med.mhv.usermgmt.report.MviUpgradePdfReport;
import gov.va.med.mhv.usermgmt.service.MviIntegrationService;

@Component
public class MviDowngradeProcessor implements ItemProcessor<InPersonAuthentication, InPersonAuthentication> {
	private static final Logger logger = LogManager.getLogger(MviDowngradeProcessor.class);

	@Autowired
	private MviIntegrationService mviIntegrationService;

	@Autowired
	private MviUpgradePdfReport pdfReport;

	// list of patients that failed to unauthenticate with MVI - used for pdf
	// report
	private List<Patient> patientErrorList = new ArrayList<Patient>();
	
	@Override
	public InPersonAuthentication process(InPersonAuthentication ipa) throws Exception {
		ResponseUtil<Object> response = new ResponseUtil<Object>();
		Patient patient;
		UserProfile userProfile;
		boolean success = false;

		try {

			patient = ipa.getPatient();

			// validation
			if (patient == null || patient.getIcn() == null) {

				if (patient == null) {
					logger.error("Not invoking Unauthentication call, patient is null, IPA id=" + ipa.getId());
				} else {
					// add patient to be used in error report
					patientErrorList.add(patient);

					logger.error("Not invoking Unauthentication call, patient ICN is null, IPA id=" + ipa.getId()
							+ ", patient id=" + patient.getId());
				}
				return null;
			}

			userProfile = patient.getUserProfile();

			// validation
			if (userProfile == null) {
				// add patient to be used in error report
				patientErrorList.add(patient);

				logger.error("Not invoking Unauthentication call, userProfile is null, patient id=" + patient.getId());
				return null;
			}

			if (logger.isInfoEnabled()) {
				logger.info("Unauthenticating patient with ICN=" + patient.getIcn() + ", userProfileId="
						+ userProfile.getId() + ", IPA id=" + ipa.getId() + ", IPA authenticationStatus="
						+ ipa.getStatus() + ", IPA mviAuthenticationStatus=" + ipa.getMviAuthenticationStatus()
						+ ", correlationStatus=" + patient.getCorrelationStatus());
			}

			mviIntegrationService.mviUnauthenticateUncorrelateForBatch(patient, userProfile, "SYSTEM", response);


			if (response.isFailure()) {

				// add patient to be used in error report
				patientErrorList.add(patient);

				if (logger.isWarnEnabled()) {
					logger.warn("Error unauthenticating patient with ICN=" + patient.getIcn() + ", userProfileId="
							+ userProfile.getId() + ", ipaId=" + ipa.getId());
					logger.warn("Failure message: " + response.getFailureMessage());

					if (logger.isDebugEnabled()) {
						Map<String, String> map = response.getInfoMessages();
						for (Map.Entry<String, String> entry : map.entrySet()) {
							logger.debug(entry.getKey() + " : " + entry.getValue());
						}
					}
				}

			} else {
				ipa.setMviAuthenticationStatus("OK");
				ipa.setStatus(AuthenticationStatusEnumeration.UNAUTHENTICATED.getDescription());
				ipa.setAuthenticationDate(null);
				success = true;

				if (logger.isInfoEnabled()) {
					logger.info("Success unauthenticating patient with ICN=" + patient.getIcn() + ", userProfileId="
							+ userProfile.getId() + ", IPA Id=" + ipa.getId() + ", IPA authenticationStatus="
							+ ipa.getStatus() + ", IPA mviAuthenticationStatus=" + ipa.getMviAuthenticationStatus());
				}
			}
		} catch (Exception e) {
			logger.error("Error in process " + e);
		}

		return success ? ipa : null;
	}
	
	@AfterStep
	public void createAuditReport(StepExecution stepExecution) {
		try {
			pdfReport.createPdfReport("mvi-downgrade-batch-job-", stepExecution.getJobExecution().getStartTime(), 
					patientErrorList, stepExecution.getReadCount());			
		} catch (Exception e) {
			logger.error("Error generating report " + e);
		} finally {
			patientErrorList.clear();
		}
	}
}
