package gov.va.med.mhv.usermgmt.batch.processor;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.batch.core.StepExecution;
import org.springframework.batch.core.annotation.AfterStep;
import org.springframework.batch.item.ItemProcessor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import gov.va.med.mhv.common.api.util.ResponseUtil;
import gov.va.med.mhv.common.data.model.InPersonAuthentication;
import gov.va.med.mhv.common.data.model.Patient;
import gov.va.med.mhv.common.data.model.UserProfile;
import gov.va.med.mhv.usermgmt.common.enums.AuthenticationStatusEnumeration;
import gov.va.med.mhv.usermgmt.report.MviUpgradePdfReport;
import gov.va.med.mhv.usermgmt.service.MviIntegrationService;

@Component
public class MviUpgradeProcessor implements ItemProcessor<InPersonAuthentication, InPersonAuthentication> {
	private static final Logger logger = LogManager.getLogger(MviUpgradeProcessor.class);

	@Autowired
	private MviIntegrationService mviIntegrationService;

	@Autowired
	private MviUpgradePdfReport pdfReport;

	// list of patients that failed to authenticate with MVI - used for pdf report
	private List<Patient> patientErrorList = new ArrayList<Patient>();

	@Override
	public InPersonAuthentication process(InPersonAuthentication ipa) throws Exception {
		ResponseUtil<Object> response = new ResponseUtil<Object>();
		Patient patient;
		UserProfile userProfile;
		boolean success = false;

		try {
			
			patient = ipa.getPatient();

			// validation
			if (patient == null) {
				logger.error("Not invoking Authenticating call, patient is null, IPA id=" + ipa.getId());
				return null;
			} else if (patient.getIcn() == null) {
				// add patient to be used in error report
				patientErrorList.add(patient);
				
				logger.error("Not invoking Authenticating call, patient ICN is null, IPA id=" + ipa.getId()
						+ ", patient id=" + patient.getId());
				return null;
			}
			
			userProfile = patient.getUserProfile();

			// validation
			if (userProfile == null) {
				// add patient to be used in error report
				patientErrorList.add(patient);

				logger.error("Not invoking Authenticating call, userProfile is null, patient id=" + patient.getId());
				return null;
			}
			
			if (logger.isInfoEnabled()) {
				logger.info("Authenticating patient with ICN=" + patient.getIcn() + ", userProfileId=" + userProfile.getId()
				+ ", IPA Id=" + ipa.getId() + ", IPA authenticationStatus=" + ipa.getStatus()
				+ ", IPA mviAuthenticationStatus=" + ipa.getMviAuthenticationStatus()
				+ ", IPA authDate=" + ipa.getAuthenticationDate());
			}

			mviIntegrationService.mviAuthenticate(patient, userProfile, true, true, response, null);

			if (response.isFailure()) {
				// add patient to be used in error report
				patientErrorList.add(patient);

				if (logger.isWarnEnabled()) {
					logger.warn("Error authenticating patient with ICN=" + patient.getIcn() + ", userProfileId="
							+ userProfile.getId() + ", ipaId=" + ipa.getId());
					logger.warn("Failure message: " + response.getFailureMessage());

					if (logger.isDebugEnabled()) {
						Map<String, String> map = response.getInfoMessages();
						for (Map.Entry<String, String> entry : map.entrySet()) {
							logger.debug(entry.getKey() + " : " + entry.getValue());
						}
					}
				}

			} else {
				ipa.setMviAuthenticationStatus("OK");
				ipa.setStatus(AuthenticationStatusEnumeration.AUTHENTICATED.getDescription());
				success = true;

				if (logger.isInfoEnabled()) {
					logger.info("Success authenticating patient with ICN=" + patient.getIcn() + ", userProfileId="
							+ userProfile.getId() + ", IPA Id=" + ipa.getId() + ", IPA authenticationStatus="
							+ ipa.getStatus() + ", IPA mviAuthenticationStatus=" + ipa.getMviAuthenticationStatus()
							+ ", IPA authDate=" + ipa.getAuthenticationDate());
				}
			}
		} catch (Exception e) {
			logger.error("Error in process " + e);
		}

		return success ? ipa : null;
	}
	
	@AfterStep
	public void afterStep(StepExecution stepExecution) {
		
		// create pdf report
		try {
			pdfReport.createPdfReport("mvi-upgrade-batch-job-", stepExecution.getJobExecution().getStartTime(), 
					patientErrorList, stepExecution.getReadCount());
		} catch (Exception e) {
			logger.error("Error generating report " + e);
		} finally {
			patientErrorList.clear();
		}

	}

}
