package gov.va.med.mhv.foundation.service.response.messages;

import gov.va.med.mhv.foundation.util.Precondition;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


public final class MessageExtUtils {
	
	// TODO 
	//   Once this class is merged into the foundation branch
	//   these methods should be added to the Message class
	//   and these method should be deprecated
	
	public static final Log LOG = LogFactory.getLog(MessageExtUtils.class);

	/**
	 * Default error text that may be used in methods that do not have a  
	 * default error text argument, but their implementation requires one.
	 */
	public static final String DEFAULT_ERROR_TEXT = 
		"An unexpected error has occurred. Please contact the helpdesk "
		+ " if the problem persists";

	/**
	 * Default Informational text that may be used in methods that do not 
	 * have a default informational text argument, but their implementation 
	 * requires one.
	 */
	public static final String DEFAULT_INFO_TEXT = null;

	public static final boolean isError(Message message) {
		return (message != null) && SeverityUtils.isError(message.
			getSeverity());
	}
	public static final boolean isError(Message message, String key) {
		return isError(message) && hasKey(message, key);
	}

	public static final boolean isInfo(Message message) {
		return (message != null) && SeverityUtils.isInformational(message.
			getSeverity());
	}
	public static final boolean isInfo(Message message, String key) {
		return isInfo(message) && hasKey(message, key); 
	}

	public static final boolean hasSeverity(Message message, 
		Severity severity) 
	{
		return (message != null) && SeverityUtils.hasSeverity(severity, 
			message.getSeverity());
	}

	public static final boolean hasKey(Message message, 
		Severity severity, String key) 
	{
		return SeverityUtils.hasSeverity(severity, message.getSeverity()) 
			&& hasKey(message, key);
	}

	protected static final boolean isKey(String key) {
		return !StringUtils.isBlank(key);
	}
	protected static final boolean hasKey(Message message, String key) {
		return (message != null) && isKey(key) && key.equals(message.getKey());
	}

	public static boolean handleMessage(Message message, 
		MessageHandler handler)
	{
		Precondition.assertNotNull("handler", handler);
		if (message == null) {
			return false;
		}
		return (handler.canHandleMessage(message))  
			? handler.handleMessage(message) : MessageExtUtils.isError(message);
	}
	
	public static boolean handleMessages(Collection<Message> messages, 
		MessageHandler handler)
	{
		Precondition.assertNotNull("handler", handler);
		if (messages == null) {
			return false;
		}
		boolean hasErrorMessage = false;
		for (Message message: messages) {
			hasErrorMessage = handleMessage(message, handler) 
				|| hasErrorMessage;
		}
		return hasErrorMessage;
	}

	public static String getText(Message message, MessageHandler handler) {
		Precondition.assertNotNull("handler", handler);
		return handler.getMessageText(message);
	}

	public static String getText(Collection<Message> messages,
		MessageHandler handler) 
	{
		return getText(messages, handler, null);
	}
	public static String getText(Collection<Message> messages, 
		MessageHandler handler, String separator) 
	{
		Precondition.assertNotNull("handler", handler);
		if (messages == null) {
			return null;
		}
		if (separator == null) {
			separator = "\n";
		}
		StringBuilder builder = new StringBuilder();
		boolean isFirst = true;
		for (Message message: messages) {
			if (isFirst) {
				isFirst = false;
			} else {
				builder.append(separator);
			}
			builder.append(getText(message, handler));
		}
		return builder.toString();
	}
	
	public static void log(Message message, Log log, Exception e) {
		if (log != null) {
			log = LOG;
		}
		if (isError(message)) {
			if (log.isErrorEnabled()) {
				if (e != null) {
					log.error(describe(message), e);
				} else {
					log.error(describe(message), e);
				}
			}
			return;
		} 
		if (isInfo(message)) {
			if (log.isInfoEnabled()) {
				if (e != null) { 
					log.info(describe(message), e);
				} else {
					log.info(describe(message));
				}
			}
		}
	}

	public static void log(Collection<Message> messages, Log log, Exception e) {
		if (messages == null) {
			return;
		}
		for (Message message: messages) {
			log(message, log, e);
		}
	}

	public static String describe(Message message) {
		if (message == null) {
			return null;
		}
		StringBuilder builder = new StringBuilder();
		builder.append(message.getKey());
		if (message.getInserts() != null) {
			builder.append(" {");
			for (Object insert: message.getInserts()) {
				builder.append((insert != null) ? insert.toString() : "");
			}
			builder.append("}");
		}
		return builder.toString();
	}

	public static String describe(Collection<Message> messages) {
		return describe(messages, null);
	}
	public static String describe(Collection<Message> messages, String newline) {
		if (messages == null) {
			return null;
		}
		if (newline == null) {
			newline = "\n";
		}
		StringBuilder builder = new StringBuilder();
		boolean first = true;
		for (Message message: messages) {
			String description = describe(message);
			if (description != null) {
				if (first) {
					first = false;
				} else {
					builder.append(newline);
				}
				builder.append(description);
			}
		}
		return builder.toString();
	}

	public static List<Object> getInsertsAsList(Message message) {
		List<Object> inserts = new ArrayList<Object>();
		if ((message != null) && (message.getInserts() != null)) {
			for (Object insert: message.getInserts()) {
				inserts.add(insert);
			}
		}
		return inserts;
	}
	
	protected MessageExtUtils() {
	}
}
